<?php
/*****************************************************************************************
  Fileame: postgresql.php

  Purpose: Contains the functions required for the wrapper to access a PostgreSQL Database

  Created: 11-01-00
  Updated: 05-02-2005 - added db_close function
  Updated: 09-01-2005 - added db_escape_string function

******************************************************************************************/

CLASS POSTGRESQL {

	private $conn;
	private $logger;

	/***********************************************************
		FUNCTION: construct
		PURPOSE:  initializes the class
		INPUTS:		dbhost -> ip address of the database host
							dbuser -> database user
							dbpassword -> database password
							dbport -> port of database
							dbname -> name of database
	***********************************************************/
	function __construct($dbhost,$dbuser,$dbpassword,$dbport,$dbname) {
		$this->conn = pg_connect("host='$dbhost' port='$dbport' user='$dbuser' password='$dbpassword' dbname='$dbname'") or die('unable to connect to postgresql!');

		//init the error handler
		if (class_exists("logger")) $this->logger = new logger($this);

	}

	/****************************************************************
		FUNCTION: getLogger()
		PURPOSE:  returns a reference to the error handler
	****************************************************************/
	function getLogger() {
		return $this->logger;
	}

	/****************************************************************
		FUNCTION: getConn()
		PURPOSE:  returns a reference to database connection
	****************************************************************/
	function getConn() {
		return $this->conn;
	}

	/****************************************************************
		FUNCTION: close()
		PURPOSE:  closes the database
	****************************************************************/
	function close() {
		pg_close($this->conn);
	}

	/****************************************************************
		FUNCTION: query()
		PURPOSE:  executes a basic sql query
		INPUTS:		sql -> string we are executing as a sql statement
		RETURNS:	pg_query result
	****************************************************************/
	function query($sql,$nolog=null) {
		if (!$sql) return false;

		if (function_exists('mb_convert_encoding')) {
			$sql = mb_convert_encoding($sql,DB_CHARSET);
		}		

		$multi = NULL;
		if (preg_match("/^COPY.* FROM stdin/i", $sql)) {
			$multi = explode( "\n", $sql );
			$sql = $multi[0] . "\n";
		}
		
		if (!$result = pg_query($this->conn,$sql)) {

			if (defined("DEBUG")) debug_print_backtrace();

			if (!$nolog) $this->logger->logerror($sql);
		}

		if ($multi) {
			for ($i = 1; $i < sizeof( $multi ); $i++) {
				$line = $multi[$i] . "\n";
				pg_put_line( $this->conn, $line );
			}
			pg_end_copy( $this->conn );
		}		

		return $result;
	
	}

	/****************************************************************
		FUNCTION: fetch()
		PURPOSE:  executes a query and returns results as an associative
							array with record[$row][$column] as format
		INPUTS:		sql -> string we are executing as a sql statement
							transpose -> causes it to return results as 
							record[$column][$row] format
		RETURNS:	array
	****************************************************************/
	function fetch($sql,$transpose=null) {

		if (!$sql) return false;

		if (!$result = pg_query($this->conn,$sql)) {
			if (defined("DEBUG")) debug_print_backtrace();
			$this->logger->logerror($sql);
			return false;
		}
	
		$num = pg_numrows($result);

		if ($num!=0) $arr = pg_fetch_all($result);

		//flip results if asked
		if ($transpose) $arr = transposeArray($arr);

		$arr["count"] = $num;
		return $arr;

	}

	/****************************************************************
		FUNCTION: single()
		PURPOSE:  executes a query and returns results as single row
							associative array
		INPUTS:		sql -> string we are executing as a sql statement
		RETURNS:	array
	****************************************************************/
	function single($sql) {

		if (!$sql) return false;
	
		if (!$result = pg_query($this->conn,$sql)) {
			if (defined("DEBUG")) debug_print_backtrace();
			$this->logger->logerror($sql);
			return false;
		}
	
		$num = pg_numrows($result);

		if ($num != 0) 
		{

			$value = @pg_fetch_assoc($result,0);
			return $value;

		}
		else return false;
	
	}

	/****************************************************************
		FUNCTION: error()
		PURPOSE:  returns last error from error handler stack
		RETURNS:	string
	****************************************************************/
	function error() {

		return $this->logger->getLastError();		
	
	}

	/****************************************************************
		FUNCTION: getAllErrors()
		PURPOSE:  returns all errors in error stack of error handler.
		INPUT:		sep -> change error delimiter for formatting
		RETURNS:	string
	****************************************************************/
	function getAllErrors($sep="html") {
		return $this->logger->getAllErrorMsgs($sep);
	}

	/****************************************************************
		FUNCTION: count()
		PURPOSE:  returns number of matches of query
		INPUT:		sql -> sql query string to run
		RETURNS:	integer
	****************************************************************/
	function count($sql) {

		if (!$result = pg_query($this->conn,$sql)) {
			$this->logger->logerror($sql);
			return false;
		}
	
		return pg_numrows($result);
	
	}

	/*********************************************
			some fun aliases
	*********************************************/
	function escort($sql,$transpose=null) {
		return $this->fetch($sql,$transpose);
	}
	
	function solo($sql) {
		return $this->single($sql);
	}

	/****************************************************************
		FUNCTION: getId()
		PURPOSE:  get the id of the primary key of the last inserted
							record 
		INPUT:		$table -> table of record
							$id -> name of column of primary key
							$result -> result from query run
		RETURNS:	integer
	****************************************************************/
	function getId($table,$id,$result) {

		//use oids to return the last inserted id for older versions of postgresql (pre 8.1)
		if (defined("USE_OID")) {

			//it didn't work, try again with the old method
			if (!$return_id) {
				$pgoid=@pg_last_oid($result);
				$result1=@pg_exec($this->conn,"SELECT $id FROM $table WHERE oid='$pgoid'");
				$query_myrow=@pg_fetch_array($result1,0);
				$return_id=$query_myrow[$id];
			}
	

		} else {
	
			$sql = "SELECT LASTVAL()";
			if ($res = @pg_exec($this->conn,$sql)) {
				$query_myrow=@pg_fetch_array($res,0);
				$return_id=$query_myrow[0];
			} 

		}

		return $return_id;
	
	}

	/*************************************************
		FUNCTION: begin()
		PURPOSE:  begins transaction instance
	*************************************************/
	function begin() {
	
		$sql = "BEGIN WORK";
		if (db_query($this->conn,$sql)) return true;
		else return false;
	}
	
	/*************************************************
		FUNCTION: end()
		PURPOSE:  ends transaction instance
	*************************************************/
	function end() {
	
		$sql = "END WORK";
		if (db_query($this->conn,$sql)) return true;
		else return false;

	}
	
	/*************************************************
		FUNCTION: vacuum()
		PURPOSE:  vacuums the database
	*************************************************/
	function vacuum() {
	
		$sql = "VACUUM FULL ANALYZE";
	
		if ($this->query($sql)) $message = "Database Vacuumed Successfully";
		else $message = "Database Vacuum Failed";
	
		return $message;
	
	}
	
	/*************************************************
		FUNCTION: escape_string()
		PURPOSE:  make string safe for db entry
	*************************************************/
	function escape_string($str) {
		return pg_escape_string($str);
	}
	
	/*************************************************
		FUNCTION: unescape_string()
		PURPOSE:  undo escape_string
	*************************************************/
	function unescape_string($str) {
	
		return str_replace("''","'",$str);
	
	}
	
	/*************************************************
		FUNCTION: increment_seq
		PURPOSE:  increments sequence and returns new value
		INPUTS:		seq -> sequence name
	*************************************************/

	function increment_seq($seq) {
	
		$sql = "SELECT NEXTVAL('".$seq."');";
		$info = $this->single($sql);
		return $info["nextval"];
	
	}
	
	/*************************************************
		FUNCTION: return_seq
		PURPOSE:  returns current value of sequence
		INPUTS:		seq -> sequence name
	*************************************************/
	function return_seq($seq) {
	
		$sql = "SELECT CURVAL('".$seq."');";
		$info = $this->single($sql);
		return $info["curval"];
	
	}
	
	/*************************************************
		FUNCTION: last_error
		PURPOSE:  returns last error of db connection
	*************************************************/
	function last_error() {
	
		return pg_last_error($this->conn);
	
	}


	/*******************************************************************
		FUNCTION: insert
		PURPOSE:  creates and executes an insert on the database
		INPUTS:		table -> table to run query on
							option -> associative array with $option[$columnname] = $newvalue setup
									query -> return the query w/o executing
									debug -> echo the query to stdout
							idField -> primary key of database.  use to return newly inserted id
	********************************************************************/
	function insert($table,$option,$idField = null,$nolog=null) {
	
		$ignoreArray = array("conn","table","debug","query","_showquery");
		$fieldArr = array();
		$valueArr = array();	

		foreach ($option AS $field=>$value) {
		
			if (in_array($field,$ignoreArray)) continue;
			if (is_numeric($field)) continue;
			if ($value==null) continue;
			
			$fieldArr[] = $field;
			$valueArr[] = "'".$value."'";
			
		}

		if (count($fieldArr) > 0 && count($valueArr) > 0) {
		
			$fieldString = implode(",",$fieldArr);
			$valueString = implode(",",$valueArr);
		
			$sql = "INSERT INTO $table (".$fieldString.") VALUES (".$valueString.");";
			if ($option["debug"]) echo $sql."<br>\n";
			if ($option["query"]) return $sql;
			if ($option["_showquery"]) file_put_contents(TMP_DIR . "/query.sql",$sql);
			
			if ($result = $this->query($sql,$nolog)) {
	
	      if ($idField) {		
	
	  			$returnId = $this->getId($table,$idField,$result);
	  			if ($returnId) return $returnId;
	  			else return true;
	
	      } else {
	        return true;
	      }
	
			} else return false;
	
		} else return false;
	
	}
	
	/*******************************************************************
		FUNCTION: update
		PURPOSE:  creates and executes an update on the database
		INPUTS:		table -> table to run query on
							option -> associative array with $option[$columnname] = $newvalue setup
									where -> which record to base your update on ($option["where"] = "id='5'")
									query -> return the query w/o executing
									debug -> echo the query to stdout
							idField -> primary key of database.  use to return newly inserted id
	********************************************************************/
	function update($table,$option,$sanitize = null) {
	
		$ignoreArray = array("conn","table","where","debug","query","_showquery");

		$queryString = null;

		foreach ($option AS $field=>$value) {
		
			if (in_array($field,$ignoreArray)) continue;
			if (is_numeric($field)) continue;
		
			if ($value==null) $queryString .= $field."=NULL,";
			else {
				if ($sanitize) 
					$queryString .= $field."='".sanitize($value)."',";
				else
					$queryString .= $field."='".$value."',";
			
			}

		}
	
		if ($queryString) {
	
			$queryString = substr($queryString,0,strlen($queryString) - 1);
				
			$sql = "UPDATE $table SET ".$queryString." WHERE ".$option["where"];
	
			if ($option["debug"]) echo $sql."<br>\n";
			if ($option["query"]) return $sql;
			if ($option["_showquery"]) file_put_contents(TMP_DIR . "/query.sql",$sql);
			
			if ($this->query($sql)) return true;
			else return false;
	
		} else return false;
	
	}

	function version()
	{
	
		$arr = pg_version();
		return $arr["server"];
	
	}

	function initial_setup($wgDBname, $wgDBuser, $wgDBpassword, $schemas, $wgDBsuperuser=null, $wgDBsuperpass=null, $dbName=null) {
		// If this is the initial connection, setup the schema stuff and possibly create the user
		print "<li>Checking the version of Postgres...";
		$version = $this->version();
		$PGMINVER = '8.4';
		if ($version < $PGMINVER) {
			die("<b>FAILED</b>. Required version is $PGMINVER. You have " . htmlspecialchars( $version ) . "</li>\n");
		}
		print "version $version is OK.</li>\n";

		$safeuser = $this->quote_ident($wgDBuser);
		// Are we connecting as a superuser for the first time?
		if ($wgDBsuperuser) {
			// Are we really a superuser? Check out our rights
			$SQL = "SELECT
				CASE WHEN usesuper IS TRUE THEN
				CASE WHEN usecreatedb IS TRUE THEN 3 ELSE 1 END
				ELSE CASE WHEN usecreatedb IS TRUE THEN 2 ELSE 0 END
				END AS rights
				FROM pg_catalog.pg_user WHERE usename = " . $this->addQuotes($wgDBsuperuser);
			$rows = pg_num_rows($res = $this->query($SQL));
			if (!$rows) {
				die("<li>ERROR: Could not read permissions for user \"" . htmlspecialchars( $wgDBsuperuser ) . "\"</li>\n");
			}
			$perms = pg_fetch_result($res, 0, 0);

			$SQL = "SELECT 1 FROM pg_catalog.pg_user WHERE usename = " . $this->addQuotes($wgDBuser);
			$rows = pg_num_rows($this->query($SQL));
			if ($rows) {
				print "<li>User \"" . htmlspecialchars( $wgDBuser ) . "\" already exists, skipping account creation.</li>\n";
			}
			else {
				if ($perms != 1 and $perms != 3) {
					die("<li>ERROR: the user \"" . htmlspecialchars( $wgDBsuperuser ) . "\" cannot create other users. Please use a different Postgres user.</li>");
				}
				print "<li>Creating user <b>" . htmlspecialchars( $wgDBuser ) . "</b>...";
				$safepass = $this->addQuotes($wgDBpassword);
				$SQL = "CREATE USER $safeuser NOCREATEDB PASSWORD $safepass";
				$this->query($SQL);
				print "OK</li>\n";
			}
			// User now exists, check out the database
			if ($dbName != $wgDBname) {
				$SQL = "SELECT 1 FROM pg_catalog.pg_database WHERE datname = " . $this->addQuotes($wgDBname);
				$rows = pg_num_rows($this->query($SQL));
				if ($rows) {
					print "<li>Database \"" . htmlspecialchars( $wgDBname ) . "\" already exists, skipping database creation.</li>\n";
				}
				else {
					if ($perms < 1) {
						die("<li>ERROR: the user \"" . htmlspecialchars( $wgDBsuperuser ) . "\" cannot create databases. Please use a different Postgres user.</li>");
					}
					print "<li>Creating database <b>" . htmlspecialchars( $wgDBname ) . "</b>...";
					$safename = $this->quote_ident($wgDBname);
					$SQL = "CREATE DATABASE $safename OWNER $safeuser ";
					$this->query($SQL);
					print "OK</li>\n";
					// Hopefully tsearch2 and plpgsql are in template1...
				}

				// Reconnect to check out tsearch2 rights for this user
				print "<li>Connecting to \"" . htmlspecialchars( $wgDBname ) . "\" as superuser \"" .
					htmlspecialchars( $wgDBsuperuser ) . "\" to check rights...";

				@$this->conn = pg_connect("host='". pg_host($this->conn) . "' port='". pg_port($this->conn) ."' user='$wgDBsuperuser' password='$wgDBsuperpass' dbname='$wgDBname'");
				if ( $this->conn == false ) {
					die("<b>FAILED TO CONNECT!</b></li>");
				}
				print "OK</li>\n";
			}

			// Setup the schema for this user if needed

			for(reset($schemas); current($schemas); next($schemas)) {
				$curschema = current($schemas);

				$result = $this->schemaExists($curschema);
				$safeschema = $this->quote_ident($curschema);
				if (!$result) {
					print "<li>Creating schema <b>" . htmlspecialchars( $curschema ) . "</b> ...";
					$result = $this->query("CREATE SCHEMA $safeschema AUTHORIZATION $safeuser");
					if (!$result) {
						die("<b>FAILED</b>.</li>\n");
					}
					print "OK</li>\n";
				}
				else {
					print "<li>Schema <b>" . htmlspecialchars( $curschema ) . "</b> already exists, explicitly granting rights...";
					$safeschema2 = $this->addQuotes($curschema);
					$SQL = "SELECT 'GRANT ALL ON '||pg_catalog.quote_ident(relname)||' TO $safeuser;'\n".
						"FROM pg_catalog.pg_class p, pg_catalog.pg_namespace n\n".
						"WHERE relnamespace = n.oid AND n.nspname = $safeschema2\n".
						"AND p.relkind IN ('r','S','v')\n";
					$SQL .= "UNION\n";
					$SQL .= "SELECT 'GRANT ALL ON FUNCTION '||pg_catalog.quote_ident(proname)||'('||\n".
						"pg_catalog.oidvectortypes(p.proargtypes)||') TO $safeuser;'\n".
						"FROM pg_catalog.pg_proc p, pg_catalog.pg_namespace n\n".
						"WHERE p.pronamespace = n.oid AND n.nspname = $safeschema2";
					$res = $this->query($SQL);
					if (!$res) {
						die("<b>FAILED</b>. Could not set rights for the user.</li>\n");
					}
					$this->query("SET search_path = $safeschema");
					$rows = pg_num_rows($res);
					while ($rows) {
						$rows--;
						$this->query(pg_fetch_result($res, $rows, 0));
					}
					print "OK</li>\n";
				}
			}

			// Install plpgsql if needed
			$this->setup_plpgsql();

			$wgDBsuperuser = '';

			return true; // Reconnect as regular user

		} // end superuser

		if (!defined('POSTGRES_SEARCHPATH')) {

			// Install plpgsql if needed
			$this->setup_plpgsql();

			for(reset($schemas); current($schemas); next($schemas)) {
				$curschema = current($schemas);

				// Does the schema already exist? Who owns it?
				$result = $this->schemaExists($curschema);
				if (!$result) {
					print "<li>Creating schema <b>" . htmlspecialchars( $curschema ) . "</b> ...";
					error_reporting( 0 );
					$safeschema = $this->quote_ident($curschema);
					$result = $this->query("CREATE SCHEMA $safeschema");
					error_reporting( E_ALL );
					if (!$result) {
						die("<b>FAILED</b>. The user \"" . htmlspecialchars( $wgDBuser ) . 
							"\" must be able to access the schema. ".
							"You can try making them the owner of the database, or try creating the schema with a ".
							"different user, and then grant access to the \"" . 
							htmlspecialchars( $wgDBuser ) . "\" user.</li>\n");
					}
					print "OK</li>\n";
				}
				else if ($result != $wgDBuser) {
					print "<li>Schema \"" . htmlspecialchars( $curschema ) . "\" exists but is not owned by \"" . 
						htmlspecialchars( $wgDBuser ) . "\". Not ideal.</li>\n";
				}
				else {
					print "<li>Schema \"" . htmlspecialchars( $curschema ) . "\" exists and is owned by \"" . 
						htmlspecialchars( $wgDBuser ) . "\". Excellent.</li>\n";
				}
			}

			// Always return GMT time to accomodate the existing integer-based timestamp assumption
			print "<li>Setting the timezone to GMT for user \"" . htmlspecialchars( $wgDBuser ) . "\" ...";
			$SQL = "ALTER USER $safeuser SET timezone = 'GMT'";
			$result = pg_query($this->conn, $SQL);
			if (!$result) {
				die("<b>FAILED</b>.</li>\n");
			}
			print "OK</li>\n";
			// Set for the rest of this session
			$SQL = "SET timezone = 'GMT'";
			$result = pg_query($this->conn, $SQL);
			if (!$result) {
				die("<li>Failed to set timezone</li>\n");
			}

			print "<li>Setting the datestyle to ISO, YMD for user \"" . htmlspecialchars( $wgDBuser ) . "\" ...";
			$SQL = "ALTER USER $safeuser SET datestyle = 'ISO, YMD'";
			$result = pg_query($this->conn, $SQL);
			if (!$result) {
				die("<b>FAILED</b>.</li>\n");
			}
			print "OK</li>\n";
			// Set for the rest of this session
			$SQL = "SET datestyle = 'ISO, YMD'";
			$result = pg_query($this->conn, $SQL);
			if (!$result) {
				die("<li>Failed to set datestyle</li>\n");
			}

			// Fix up the search paths if needed
			print "<li>Setting the search path for user \"" . htmlspecialchars( $wgDBuser ) . "\" ...";
			$path = "";
			for(reset($schemas); current($schemas); next($schemas)) {
				if($path) $path .= ", ";
				$path .= $this->quote_ident(current($schemas));
			}

			$SQL = "ALTER USER $safeuser SET search_path = $path";
			$result = pg_query($this->conn, $SQL);
			if (!$result) {
				die("<b>FAILED</b>.</li>\n");
			}
			print "OK</li>\n";
			// Set for the rest of this session
			$SQL = "SET search_path = $path";
			$result = pg_query($this->conn, $SQL);
			if (!$result) {
				die("<li>Failed to set search_path</li>\n");
			}
			define( "POSTGRES_SEARCHPATH", $path );
		}
	}

	function addQuotes( $s ) {
		if ( is_null( $s ) ) {
			return 'NULL';
		} else if ( is_bool( $s ) ) {
			return intval( $s );
		}
		return "'" . pg_escape_string($s) . "'";
	}

	function quote_ident( $s ) {
		return '"' . preg_replace( '/"/', '""', $s) . '"';
	}

	function schemaExists( $schema ) {
		$owner = false;
		$eschema = preg_replace("/'/", "''", $schema);
		$SQL = "SELECT rolname FROM pg_catalog.pg_namespace n, pg_catalog.pg_roles r "
			."WHERE n.nspowner=r.oid AND n.nspname = '$eschema'";
		$res = $this->single( $SQL );
		if( $res && array_key_exists("rolname", $res) )
			$owner = $res["rolname"];
		return $owner;
	}

	function setup_plpgsql() {
		print "<li>Checking for Pl/Pgsql ...";
		$SQL = "SELECT 1 FROM pg_catalog.pg_language WHERE lanname = 'plpgsql'";
		$rows = pg_num_rows($this->query($SQL));
		if ($rows < 1) {
			// plpgsql is not installed, but if we have a pg_pltemplate table, we should be able to create it
			print "not installed. Attempting to install Pl/Pgsql ...";
			$SQL = "SELECT 1 FROM pg_catalog.pg_class c JOIN pg_catalog.pg_namespace n ON (n.oid = c.relnamespace) ".
				"WHERE relname = 'pg_pltemplate' AND nspname='pg_catalog'";
			$rows = pg_num_rows($this->query($SQL));
			if ($rows >= 1) {
				$olde = error_reporting(0);
				error_reporting($olde - E_WARNING);
				$result = $this->query("CREATE LANGUAGE plpgsql");
				error_reporting($olde);
				if (!$result) {
					die("<b>FAILED</b>. You need to install the language plpgsql in the database <tt>" . 
						htmlspecialchars( $wgDBname ) . "</tt></li>");
				}
			}
			else {
				die("<b>FAILED</b>. You need to install the language plpgsql in the database <tt>" . 
					htmlspecialchars( $wgDBname ) . "</tt></li>");
			}
		}
		print "OK</li>\n";
	}

	function setup_database($user, $schemas) {

		// Make sure that we can write to the correct schema
		// If not, Postgres will happily and silently go to the next search_path item
		for(reset($schemas); current($schemas); next($schemas)) {
			$curschema = current($schemas);

			$ctest = "docmgr_test_table";
			$safeschema = $this->quote_ident($curschema);
			if ($this->tableExists($ctest, $curschema)) {
				$this->query("DROP TABLE $safeschema.$ctest");
			}
			$SQL = "CREATE TABLE $safeschema.$ctest(a int)";
			$olde = error_reporting( 0 );
			$res = $this->query($SQL);
			error_reporting( $olde );
			if (!$res) {
				die("<b>FAILED</b>. Make sure that the user \"" . htmlspecialchars( $user ) . 
					"\" can write to the schema \"" . htmlspecialchars( $curschema ) . "\"</li>\n");
			}
			$this->query("DROP TABLE $safeschema.$ctest");
		}

		$this->query("BEGIN");
		$res = $this->sourceFile("install/docmgr.pgsql");

		$this->query("COMMIT");
	}

	function relationExists( $table, $types, $schema ) {
		if ( !is_array( $types ) )
			$types = array( $types );
		$etable = $this->addQuotes( $table );
		$eschema = $this->addQuotes( $schema );
		$SQL = "SELECT 1 FROM pg_catalog.pg_class c, pg_catalog.pg_namespace n "
			. "WHERE c.relnamespace = n.oid AND c.relname = $etable AND n.nspname = $eschema "
			. "AND c.relkind IN ('" . implode("','", $types) . "')";
		$res = $this->query( $SQL );
		$count = $res ? pg_num_rows($res) : 0;
		if ($res)
			pg_free_result( $res );
		return $count ? true : false;
	}

	function tableExists( $table, $schema ) {
		return $this->relationExists( $table, array( 'r', 'v' ), $schema );
	}

	function sourceFile( $filename, $lineCallback = false, $resultCallback = false ) {
		$fp = fopen( $filename, 'r' );
		$cmd = "";
		$done = false;
		$dollarquote = false;
		$copystdin = false;

		while ( ! feof( $fp ) ) {
			if ( $lineCallback ) {
				call_user_func( $lineCallback );
			}
			## Strip away all the regular characters except for tabs which are used as delimiter needed for copystdin
			$line = trim( fgets( $fp, 1024 ), "\x20\x0A\x0D\x00\x0B");
			$sl = strlen( $line ) - 1;

			if ( $sl < 0 ) { continue; }
			if ( '-' == $line{0} && '-' == $line{1} ) { continue; }

				## Allow dollar quoting for function declarations
				if (substr($line,0,4) == '$dm$') {
					if ($dollarquote) {
						$dollarquote = false;
						$done = true;
					}
					else {
						$dollarquote = true;
					}
				}
				else if (preg_match("/^COPY.* FROM stdin/i", $line)) {
					$copystdin = true;
				}
				else if (!$dollarquote && !$copystdin) {
					if ( ';' == $line{$sl} && ($sl < 2 || ';' != $line{$sl - 1})) {
						$done = true;
						$line = substr( $line, 0, $sl );
					}
				}

			if ( '' != $cmd && !$copystdin ) { $cmd .= ' '; }
				$cmd .= "$line\n";

			if ($copystdin && substr($line,0,2) == '\.') {
				$copystdin = false;
				$done = true;
			}

			if ( $done ) {
				$cmd = str_replace(';;', ";", $cmd);
				$res = $this->query( $cmd, __METHOD__ );
				if ( $resultCallback ) {
					call_user_func( $resultCallback, $res, $this );
				}

				if ( false === $res ) {
					$err = pg_last_error();
					fclose($fp);
					return "Query \"{$cmd}\" failed with error code \"$err\".\n";
				}

				$cmd = '';
				$done = false;
			}
		}
		fclose($fp);
		return true;
	}		

}
