<?php

/**
 * PDO CalDAV backend
 *
 * This backend is used to store calendar-data in a PDO database, such as 
 * sqlite or MySQL
 * 
 * @package Sabre
 * @subpackage CalDAV
 * @version $Id: PDO.php 902 2010-02-20 10:13:44Z evertpot@gmail.com $
 * @copyright Copyright (C) 2007-2010 Rooftop Solutions. All rights reserved.
 * @author Evert Pot (http://www.rooftopsolutions.nl/) 
 * @license http://code.google.com/p/sabredav/wiki/License Modified BSD License
 */
class Sabre_CalDAV_Backend_DOCMGR extends Sabre_CalDAV_Backend_Abstract {

    /**
     * pdo 
     * 
     * @var PDO
     */
    private $DOCMGR;

    /**
     * List of CalDAV properties, and how they map to database fieldnames
     *
     * Add your own properties by simply adding on to this array
     * 
     * @var array
     */
    public $propertyMap = array(
        '{DAV:}displayname'                          => 'displayname',
        '{urn:ietf:params:xml:ns:caldav}description' => 'description',
        '{http://apple.com/ns/ical/}calendar-order'  => 'calendarorder',
        '{http://apple.com/ns/ical/}calendar-color'  => 'calendarcolor',
    );

    /**
     * Creates the backend 
     * 
     * @param PDO $pdo 
     */
    public function __construct() {

    }

    /**
     * Returns a list of calendars for a principal
     *
     * @param string $userUri 
     * @return array 
     */
    public function getCalendarsForUser($principalUri) {

        global $DOCMGR;

        $opt = null;
        $opt["command"] = "calendar_ical_getcalendars";
        $opt["caldav"] = "t";
        $arr = $DOCMGR->call($opt);

        $len = count($arr["calendar"]);

        $calendars = array();
        
        for ($i=0;$i<$len;$i++)
        {

          $row = $arr["calendar"][$i];

          //rename some stuff
          $row["principaluri"] = $row["principal_uri"];
          $row["displayname"] = $row["display_name"];
          $row["calendarcolor"] = $row["color"];
                 
          //remap some fields        
          if (!$row["calendarorder"]) $row["calendarorder"] = $i+1;
          if (!$row["calendarcolor"]) $row["calendarcolor"] = "#2CA10BFF";

          $calendar = array(
                'id' => $row['id'],
                'uri' => $row['uri']
          );

          foreach($this->propertyMap as $xmlName=>$dbName) 
          {
            $calendar[$xmlName] = $row[$dbName];
          }

          $calendars[] = $calendar;
        
        }

        return $calendars;

    }

    /**
     * Creates a new calendar for a principal.
     *
     * If the creation was a success, an id must be returned that can be used to reference
     * this calendar in other methods, such as updateCalendar
     *
     * @param string $principalUri
     * @param string $calendarUri
     * @param array $properties
     * @return mixed
     */
    public function createCalendar($principalUri,$calendarUri, array $properties) {

        global $DOCMGR;

        $fieldNames = array(
            'principaluri',
            'uri',
        );
        $values = array(
            ':principaluri' => $principalUri,
            ':uri'          => $calendarUri
        );

        foreach($this->propertyMap as $xmlName=>$dbName) {
            if (isset($properties[$xmlName])) {
                $values[':' . $dbName] = $properties[$xmlName];
                $fieldNames[] = $dbName;
            }
        }


        $num = count($fieldNames);
        $keys = array_keys($values); 

        $opt = array();
                                
        for ($i=0;$i<$num;$i++) $opt[$fieldNames[$i]] = $keys[$i];
        
        $opt = $this->reformatData($data);
        $opt["command"] = "calendar_ical_savecalendar";
        $opt["caldav"] = "t";
        $arr = $DOCMGR->call($opt);
        
        return $arr["calendar_id"];
        
    }


    /**
     * Updates a calendar's properties
     *
     *
     * The mutations array has 3 elements for each item. The first indicates if the property
     * is to be removed or updated (Sabre_DAV_Server::PROP_REMOVE and Sabre_DAV_Server::PROP_SET)
     * the second is the propertyName in Clark notation, the third is the actual value (ommitted
     * if the property is to be deleted).
     *
     * The result of this method should be another array. Each element has 2 subelements with the 
     * propertyname and statuscode for the change
     *
     * For example:
     *   array(array('{DAV:}prop1',200), array('{DAV:}prop2',200), array('{DAV:}prop3',403))
     *
     * The default implementation does not allow any properties to be updated, and thus
     * will return 403 for each one.
     *
     * @param string $calendarId
     * @param array $mutations
     * @return array 
     */
    public function updateCalendar($calendarId, array $mutations) {

        global $DOCMGR;

        $values = array();

        $result = array();

        foreach($mutations as $mutation) {

            // If the fieldname is not in the propertymap, we deny the update
            if (!isset($this->propertyMap[$mutation[1]])) {
                $result[] = array($mutation[1],403);
                continue;
            }
            $value = $mutation[0]===Sabre_DAV_Server::PROP_REMOVE?null:$mutation[2];
            $fieldName = $this->propertyMap[$mutation[1]];

            $values[$fieldName] = $value;

            // We're assuming early that the property update will succeed
            // if it doesn't, we'll get a SQL error anyway.
            $result[] = array($mutation[1],200);


        }
       
        // If the values array is empty, it means no supported
        // field are updated. The result should only contain 403 statuses
        if (count($values)===0) return $result;
    
        $opt = null;
            
        foreach($values as $fieldName=>$value) {
            $opt[$fieldName] = $value;
        }

        $opt = $this->reformatData($opt);
        $opt["command"] = "calendar_ical_savecalendar";
        $opt["calendar_id"] = $calendarId;
        $opt["caldav"] = "t";
        $arr = $DOCMGR->call($opt);

        return $result;

    }

    //reformats his fields to our database
    private function reformatCalData($data)
    {
    
      if ($data["principaluri"]) 
      {
        $data["principal_uri"] = $data["principaluri"];
        unset($data["principaluri"]);
      }        

      if ($data["displayname"]) 
      {
        $data["display_name"] = $data["displayname"];
        unset($data["displayname"]);
      }        

      if ($data["calendarcolor"]) 
      {
        $data["color"] = $data["calendarcolor"];
        unset($data["calendarcolor"]);
      }        

      return $data;
    
    }

    /**
     * Delete a calendar and all it's objects 
     * 
     * @param string $calendarId 
     * @return void
     */
    public function deleteCalendar($calendarId) {

      global $DOCMGR;
      
      $opt = null;
      $opt["command"] = "calendar_ical_deletecalendar";
      $opt["calendar_id"] = $calendarId;
      $DOCMGR->call($opt);

    }

    /**
     * Returns all calendar objects within a calendar object. 
     * 
     * @param string $calendarId 
     * @return array 
     */
    public function getCalendarObjects($calendarId) {

        global $DOCMGR;

        $opt = null;
        $opt["command"] = "calendar_ical_getevents";
        $opt["calendar_id"] = $calendarId;
        $arr = $DOCMGR->call($opt);

        $event = array();
        $num = count($arr["event"]);
        
        for ($i=0;$i<$num;$i++)
        {
        
          $row = $arr["event"][$i];
          $row["calendardata"] = $row["data"];
          $row["calendarid"] = $row["calendar_id"];
          $row["lastmodified"] = $row["last_modified"];

          $event[] = $row;
          
        }

        return $event;

    }

    /**
     * Returns information from a single calendar object, based on it's object uri. 
     * 
     * @param string $calendarId 
     * @param string $objectUri 
     * @return array 
     */
    public function getCalendarObject($calendarId,$objectUri) {

        global $DOCMGR;

        $opt = null;
        $opt["command"] = "calendar_ical_getevents";
        $opt["calendar_id"] = $calendarId;
        $opt["uri"] = $objectUri;
        $arr = $DOCMGR->call($opt);
       
        if ($arr["event"]) 
        {
        
          $row = $arr["event"][0];
          $row["calendardata"] = $row["data"];
          $row["calendarid"] = $row["calendar_id"];
          $row["lastmodified"] = $row["last_modified"];

          return $row;

        } else return array();

    }

    /**
     * Creates a new calendar object. 
     * 
     * @param string $calendarId 
     * @param string $objectUri 
     * @param string $calendarData 
     * @return void
     */
    public function createCalendarObject($calendarId,$objectUri,$calendarData) {

        global $DOCMGR;

        $opt = null;
        $opt["command"] = "calendar_ical_saveevent";
        $opt["calendar_id"] = $calendarId;
        $opt["uri"] = $objectUri;
        $opt["create"] = 1;
        $opt["data"] = $calendarData;
        $arr = $DOCMGR->call($opt);

    }

    /**
     * Updates an existing calendarobject, based on it's uri. 
     * 
     * @param string $calendarId 
     * @param string $objectUri 
     * @param string $calendarData 
     * @return void
     */
    public function updateCalendarObject($calendarId,$objectUri,$calendarData) {

        global $DOCMGR;

        $opt = null;
        $opt["command"] = "calendar_ical_saveevent";
        $opt["calendar_id"] = $calendarId;
        $opt["update"] = 1;
        $opt["uri"] = $objectUri;
        $opt["data"] = $calendarData;
        $arr = $DOCMGR->call($opt);

    }

    /**
     * Deletes an existing calendar object. 
     * 
     * @param string $calendarId 
     * @param string $objectUri 
     * @return void
     */
    public function deleteCalendarObject($calendarId,$objectUri) {

        global $DOCMGR;

        $opt = null;
        $opt["command"] = "calendar_ical_deleteevent";
        $opt["calendar_id"] = $calendarId;
        $opt["uri"] = $objectUri;
        $arr = $DOCMGR->call($opt);

    }


}
